use core::fmt::{self};

/// A canonical 32-bit virtual memory address.
#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
#[repr(transparent)]
pub struct VirtAddr(u32);

impl VirtAddr {
	/// Create a new [`VirtAddr`] from a [`u32`].
	#[inline]
	pub const fn new(addr: u32) -> Self {
		Self(addr)
	}

	/// Converts this address to a `u32`.
	#[inline]
	pub const fn as_u32(self) -> u32 {
		self.0
	}
}

impl fmt::Debug for VirtAddr {
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
		f.debug_tuple("VirtAddr")
			.field(&format_args!("{:#x}", self.0))
			.finish()
	}
}

impl fmt::Binary for VirtAddr {
	#[inline]
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
		fmt::Binary::fmt(&self.0, f)
	}
}

impl fmt::LowerHex for VirtAddr {
	#[inline]
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
		fmt::LowerHex::fmt(&self.0, f)
	}
}

impl fmt::Octal for VirtAddr {
	#[inline]
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
		fmt::Octal::fmt(&self.0, f)
	}
}

impl fmt::UpperHex for VirtAddr {
	#[inline]
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
		fmt::UpperHex::fmt(&self.0, f)
	}
}
